;--------------------------------------------------
;
; Profi-Monitor V5.5
; (C)hms85 Reinhard Gößler  1985
;                              
; Neu aufgebaut W.Römer 2015
; ich habe mich möglichst an die orginalen Labelnamen gehalten
; Sprünge inerhalb des Monitors sind mit label_m versehen
; Sprünge ausserhalb sind mit label_V für Video-Monitor
;		         bzw. label_F für FDC-Routinen
;
; Labels die noch nicht bestimmt wurden sind durchnummeriert
; unbenutzte sind auskommentiert
;
; Mit "8085 Simulator IDE" assembliert
;
; Stand: 20.01.2015 - Analyse V24-Schnittstelle
; Stand: 05.01.2015 - symbolische Adressen
;
;---------------------------------------------------
; Constante/Definitionen
;
;
rst5on equ  1eh  ;8085
rstcon equ  02h  ;NSC800
rst6on equ  1dh
rstbon equ  04h
rst7on equ  1bh
rstaon equ  08h
rstoff equ  17h
nscoff equ  00h
;
;---------------------------------------------------------
;
; IO-Adressen
;
; V24-Schnittstellenkarte
;
ACIA_CTL	EQU  0E9h		; ACIA 6850 Control/Statusregister
ACIA_DAT	EQU  0F9H		;           Daten lesen/schreiben
;
CS_MODE		EQU  0A9H		; Modus (Eingangswahlschalter)
CS_BELL		EQU  0B9H		; Bell  (Schnarre)
;
TIMER_CTL	EQU  0D9H		; Timer 8253 Controlregister
TIMER_0		EQU  89H		;            Zähler 0
TIMER_1		EQU  99h		;	     Zähler 1	* nicht benutzt
TIMER_2		EQU  0C9h		;	     Zähler 2	* nicht benutzt
;
; FDC-Karte
;
FDC_SEL		EQU  40H		; Floppy Selekt-Byte
FDC_CTL		EQU  48H		; WD1720 Controlregister
FDC_SP		EQU  49H		; Spurregister
FDC_SEK		EQU  4AH		; Sektorregister
FDC_DAT		EQU  4BH		; Datenregister
;
; Parellelschnittstelle
;
PAR_CTL		EQU  0D8H		; 8255 Control/Statusregister
PAR_PA		EQU  0A8H		;      Port A
PAR_PB		EQU  0B8H		;      Port B
PAR_PC		EQU  0C8H		;      Port C
;
; Echtzeituhr
;
UHR_CTL		EQU  19H		; 5832 Controlregister
UHR_DAT		EQU  18H		;      Datenregister
;
; EPROM Brenner
;
PROM_CTL	EQU  0BH		; Controlregister
PROM_ADRL	EQU  09H		; Adressen untere Hälfte
PROM_ADRH	EQU  0AH		;          obere Hälfte
PROM_DAT	EQU  08H		; Daten
;
; Einzelschrittmodul
;
SSM_FF		EQU  1AH		; Singlestep-Modul FlipFlop
;
; Videokarte
;
VID_DSPEN 	equ  2ah  		; Format/Blanking (Umschaltung Zeichensatz/ Abfrage DiSPlayENable
;
; HEX-Display
;
LED_ZEIL	EQU  00H		; LED-Zeile
HEX_D1		EQU  10H		; DIGIT 1
HEX_D2		EQU  20H		; DIGIT-2
HEX_D3		EQU  30H		; DIGIT-3
HEX_D4		EQU  40H		; DIGIT-4
HEX_D5		EQU  50H		; DIGIT-5
HEX_D6		EQU  60H		; DIGIT-6
HEX_D7		EQU  70H		; DIGIT-7
;
;---------------------------------------------------------
;
; bis  2DFFh     :Anwender-RAM-Bereich
; ab 2E00h Buffer-Adressen
;
; Adressen hier zusammengefasst
;
buff_th:	equ	2e00h     	; Buffer f.Thermo-Drucker
buff_fkt:	equ	2e20h		; Fct-Tasten
buff_vid:	equ	2e4eh 		; Video
buff_ln:   	equ	2e60h     	; Linebuffer 80 Zeichen
buff_mon:	equ	2eb0h		; Monitor
buff_sta:	equ	2f00h		; Stack
buff_ass:	equ	2f00h		; Assembler
buff_usr:	equ	2f70h		; Userstack
buff_fdc:	equ	2f70h		; FDC-Routinen
buff_irq:	equ	2f8ch		; Interuptverteiler
buff_uhr:	equ	2fdch		; Uhr-BCD
buff_asc:	equ	2feah 		; Uhr-ASCII
;
STACK	EQU 2F00H	; 
; RST551	EQU 2F99h
;
; HORDSP	EQU 2E50h		; 40/80 Zeichen pro Zeile
; BEGIN	EQU 2E54h			; Anfangsadresse letzte Zeile
UPDATE	EQU 2E56h	; aktuelle Cursorposition
;
SRCBEG	EQU 2FBFh	; Startadresse
SRCEND	EQU 2FC1h	; Endadresse
DSTBEG	EQU 2FC3h	; Anfang Zieladresse/Parameter
DSTEND	EQU 2FC5h	; 
RAMBEG	equ 37B0h	;
; LINZ	EQU 2E54h
;
VIDBEG	EQU 3000h	; Bildschirmadresse 	oben links
VIDEND	EQU 377Fh	; 			unten rechts
; PRTFLG	EQU 37AFh		; 
; RAMBEG	EQU 37B0h		; Anfang Parameterbereich Videomonitor
;
; Speicherzellen im RAM teilweise durchnummeriert
;
REG_A	equ	2FA7H	; Inhalt Reg A
REG_B	equ	2FA8H	;      	 Reg B
REG_C 	equ	2FA9h	;	 Reg C
REG_D	equ	2FAAh	;	 Reg D 
REG_E	equ	2FABh	;	 Reg E 
REG_F	equ	2FACH	; 	 Reg F 
REG_H	equ	2FADh 	;	 Reg H
REG_L	equ	2FAEh	;	 Reg L
REG_SP	equ	2FAFH	;	 Reg Stackpointer & Programmpointer
;

R_106	equ	2FC7H	;
R_107	equ	2FC8H	;
R_100	equ	2FCBH	; 
R_115	equ	2FCDh	;


R_108	equ	2FB2H	;
R_109	equ	2FB3H	;
R_110	equ	2FB6H	;
R_111	equ	2FB9H	;
R_112	equ	2FBBH	;
R_113	equ	2FBCH	;
R_105	equ	2FBDH	;


;R_116	equ
;R_117	equ
;R_118	equ

;
; Sprungtabelle für Funktionstasten
;
FCT_7	EQU	2E20h	;
FCT_6	EQU	2E22h	;
FCT_5	EQU	2E24h	;
FCT_4	EQU	2E26h	;
FCT_3	EQU	2E28h	;
FCT_2	EQU	2E2Ah	;
FCT_1	EQU	2E2Ch	;
FCT_97	EQU	2E2Eh	; 87er Hardware
FCT_77	EQU	2E30h	; 87er Hardware
FCT_DEL	EQU	2E32h	;
FCT_PL	EQU	2E34h	;
FCT_SS	EQU	2E36h	;
FCT_0	EQU	2E38h	;
FCT_9 	EQU	2E3Ah	;
FCT_8	EQU	2E3Ch	;
FCT_87	EQU	2E3Eh	; 87er Hardware

;
; Buffer Adressen
;
BATBUF	EQU 2D00h
FDCBUF	EQU 2F70h	; bzw. UserStack
CPYBUF	EQU 8000h
;
; 
; 
;----------------------------------------------
;
	Org	0000H		; ROM-Platz #0 auf CPU-Karte
	
        JMP KALTST		; RST0	--> Kaltstart
WARMST: JMP WARM		;	--> Warmstart
	DB  06h			; Taktfrequenz in MHz 
        DB  04h			; Prellzeit der Tastatur in ms
RST1:  	JMP RST1_m		; 
DELY1:	JMP DELY1_m		; DELY1 --> 1ms Laufzeit
	DB  00h			; Baudrate für Drucker
	DB  48H			; 4800 Bd
	JMP RST2_m		; RST2
DELY100: JMP DELY100_m		; DELY100 --> 100ms Laufzeit
	DB  00h			; Baudrate für Cassetten
	DB  12H			; 1200Bd
	JMP RST3_m		; RST3
ONESEC: JMP ONESEC_m		; ONESEC --> 1s Laufzeit
	DB  00h			; Baudrate für V24-Schnittstelle
	DB  48H			; 4800Bd
	JMP RST4_m		; RST4
	DB  00h			; 
	PCHL			; TRAP	--> FDC-Routinen
	JMP TRAP_m
	JMP RST5_m		; RST5
	NOP
	JMP RST55_m		; RST5.5 
	NOP
	JMP RST6_m		; RST6
	NOP
	JMP RST65_m		; RST6.5 
	NOP
	JMP RST7_m		; RST7
	NOP
	JMP RST75_m		; RST7.5 
	MOV D,L
	JMP L0017
CI:  	JMP CI_V		; CI ASCII-Tastatur abfragen (A)
RI:	JMP RI_m		; RI ein Byte von Kasetteninterface empfangen (A)
CO:	JMP CO_m		; CO Inhalt(C) ASCII an Bildschirm (UPDATE)
PO:	JMP PO_m		; PO Inalt (C) auf Kasetteninterface (1200Bd)
LO:	JMP LO_m		; LO Inalt (C) ASCII-Zeichen an serielle Scnitstelle (4800Bd)
CSTS:  	JMP CSTS_V		; CSTS Status ASCII-Tastatur gedrückt Z=0 sonst Z=1
	JMP L0024
	JMP L0025
MEMCHK:	JMP MEMCHK_m		; MEMCHK Speicertest (AB)max Adressen
	DB  14H			; Druckerbetriebsart (8bit,1Stop,ohne Parität)
	db  10h           	; Cassettenbetriebsart (8daten, ohne Parität)
	DB  14H			; V24 Betribsart (8bit,1Stop,ohne Parität)
TI:	JMP TI_m		; TI Uhrzeit lesen ASCII in Buffer 
	DB  32h			;
	DB  0BH			;
	DB  0E9h			;
L0173:  JMP L0028
CASOT:	JMP CASOT_m		; CASOT Kasettenausgabe (HL)Start (DE)Länge
CASIN:  JMP CASIN_m		; CASIN Kasetteneingabe (HL)Start (DE)Länge
UHR:	JMP UHR_m		; UHR Uhrzeitbuffer + 1 
WRPROM: JMP WRPROM_m		; WRPROM EPROM programmieren (A)Typ, (HL)Start Quelle, (DE)Ende Quelle, (BC)StartEPROM
RDPROM: JMP RDPROM_m		; RDPROM EPROM lesen (A)Typ, (HL)StartEPROM, (DE)EndeEPROM, (BC)Start Zieladresse
WRITUR:	JMP WRITU_m		; WRITUR Uhrzeit stellen (HL)Std,min, (BC)Wochentag, (DE)Monat,Jahr
READUR:	JMP READUR_m		; READUR Uhrzeit lesen BCD(HL)Adresse 
PRTHEX: JMP PRTHEX_m		; PRTHEX Thermo-Drucker (HL)Startadresse (DE)Endadresse
PRTASC:	JMP PRTASC_m		; PRTASC Thermo-Drucker (HL)Start Textbuffer, (SRCEND) Zeilenzahl
FILL:	JMP FILL_M		; FILL Speicher füllen (SRCBEG,SRCEND,DSTBEG)
COPY: 	JMP COPY_m		; COPY Speicher kopieren (SRCBEG,SRCEND,DSTBEG)
	JMP L0040
	JMP L0041
	JMP L0042
	JMP L0043
V24OUT:	JMP V24OUT_m		; V24OUT über CPU-Seriellescnnittstelle (HL)StartQuelle, (DE)EndeQuelle, (BC)Baudrate 
V24IN	JMP V24IN_m		; V24IN  über CPU-Seriellescnnittstelle (HL)StartZiel, (DE)EndeZiel, (BC)Baudrate
DIPL2:  JMP DIPL2_m		; DISPL2 Akku 2stellig LED-Zeile
DIPL4:	JMP DIPL4_m		; DISPL4 HL 4stellig in LED-Zeile
	JMP L0048
	JMP L0049
	JMP L0050
	JMP L0051
	JMP L0052
	JMP L0053
	JMP L0054
	JMP L0055
	JMP L0056
	JMP L0057
	JMP L0058
	JMP L0059
	JMP L0060
	JMP L0061
	JMP L0062		;
;		
; Ende der Sprungtabelle		
;
KALTST: LXI SP,STACK		;
	CALL PRTHEX_m
	CALL L0063
	LXI H,R_100
	MVI M,0FFH
	IN 0BCH
	CPI 7FH
	JZ L0064
	INR M
	CPI 80H
	JZ L0064
	CPI 00H
	JZ KALT_V		;
	DCR M
	JMP KALT_V		;
	CALL L0066
WARM:  	XRA A			;
	OUT FDC_SEL
	LXI SP,STACK
L0064:  CALL L0067		;
	CALL L0068
	CALL L0069
	JMP L0064
L0063:  LXI H,FDCBUF		;
	MOV A,L
	MOV L,H
	MOV H,A
	SHLD REG_SP
	MVI A,00H
	STA 0000H
	MVI A,00H
	STA R_105
	STA R_107
	CALL L0070
	CALL L0071
	MVI A,00H
	STA R_106
	OUT SSM_FF		; Einzelschrittmodul FlipFlop
	OUT 40H			; FDC Selektbyte
	OUT LED_ZEIL
	OUT PROM_CTL
	MVI A,0CH
	OUT 38H
	MVI A,20H
	OUT CS_MODE
	MVI A,82H
	OUT PAR_CTL
	MVI A,0F0H
	OUT PROM_ADRH
	MVI A,0F4H
	OUT UHR_CTL
	MVI A,17H
	SIM
	NOP
	IN FDC_DAT
	OUT CS_BELL
	RET
;
PRTHEX_m:  STA REG_A		;
	MOV A,L
	MOV L,H
	MOV H,A
	SHLD REG_H
	PUSH PSW
	POP H
	MOV A,L
	STA REG_F
	LXI H,REG_B
	MOV M,B
	INX H
	MOV M,C
	INX H
	MOV M,D
	INX H
	MOV M,E
	LXI H,0000H
	DAD SP
	INX H
	INX H
	MOV A,L
	MOV L,H
	MOV H,A
	SHLD REG_SP
	RET			;
;
L0067:  CALL UHR_m
	RNC
	CALL L0072
	CALL L0073
	CALL CASOT_m
	RET
;
UHR_m:  CALL L0074
	RNC
	MOV B,A
	CALL L0075
	CALL L0074
L0101:  CMP B
	JZ L0076
	STC
	CMC
	RET
	;
L0076:  STC
	RET
;
L0074:  IN 0ECH
	CALL L0077
	RC
	IN 0DCH
	MVI C,08H
	CALL L0078
	RET
	;
L0077:  MVI C,00H
L0078:  LXI H,R_100
	XRA M
	RZ
L0080:  RRC
	JC L0079
	INR C
	JMP L0080
L0079:  MOV A,C
	RET
;
L0072:  LXI H,2FBAH
	LDA R_105
	RRC
	JC L0081
	RRC
	JC L0082
	RET
;	
L0082:  RRC
	JC L0083
	INX H
	CALL L0084
	MOV A,M
	MOV M,C
	ANI 0F0H
	RRC
	RRC
	RRC
	RRC
	MOV C,A
	INX H
L0081:  CALL L0084
	MOV M,C
	LDA R_105
	MOV B,A
	ANI 08H
	RZ
	MOV A,B
	ANI 0FEH
	ORI 02H
	STA R_105
	RET
;
L0083:  MOV A,C
	ADI 06H
	ANI 07H
	STA R_106
	RET
;
L0084:  MOV A,M
	ANI 0FH
	RLC
	RLC
	RLC
	RLC
	ORA C
	MOV C,A
	RET
;
L0109:  LDA R_105
	MOV B,A
	ANI 08H
	RNZ
	MOV A,B
	ANI 04H
	JNZ L0085
	LHLD R_112
L0086:  MOV A,M
	STA 2FBAH
	RET
;
L0085:  LXI H,REG_A
	LDA R_106
	MOV E,A
	MVI D,00H
	DAD D
	JMP L0086
L0111:  LDA R_105
	ANI 04H
	JNZ L0087
	LHLD R_112
L0088:  LDA 2FBAH
	MOV M,A
	RET
;
L0087:  LXI H,REG_A
	LDA R_106
	MOV E,A
	MVI D,00H
	DAD D
	JMP L0088
L0073:  LDA 2FBAH
	MOV B,A
	LDA R_105
	ANI 80H
	JZ L0089
	LDA REG_F
	MOV B,A
L0089:  MOV A,B
	STA R_108
	LXI D,R_109
	LDA 2FBAH
	CALL L0028
	LXI D,R_110
	LDA R_105
	ANI 04H
	JNZ L0090
	LDA R_112
	CALL L0028
	LDA R_113
	CALL L0028
	JMP L0091
L0090:  LDA R_106
	LXI H,02BCH
	CALL L0092
	XRA A
	STAX D
	INX D
	STAX D
	INX D
	STAX D
L0091:  LXI D,R_109
	LDA R_105
	ANI 01H
	JNZ L0093
	INX D
	INX D
	INX D
L0093:  LDAX D
	ORI 80H
	STAX D
	RET
	;
L0028:  MOV B,A
	CALL L0094
	MOV A,B
	RRC
	RRC
	RRC
	RRC
	CALL L0094
	RET
;
L0094:  LXI H,02B2H		; Tabellenadresse laden
L0092:  PUSH D
	MVI D,00H
	ANI 0FH
	MOV E,A
	DAD D
	POP D
	MOV A,M
	STAX D
	INX D
	RET
;
; Tabelle ?? Adresse 02B2H
;
	DB 3Fh			; 
	DB 06h			;
	DB 5Bh			;
	DB 4Fh			;
	DB 66h			;
	DB 6Dh			;
	DB 7Dh			;
	DB 07h                  ;
	DB 7Fh                  ;
	DB 6Fh			;
	DB 77h			;
	DB 7Ch			;
	DB 39h			;
	DB 5Eh			;
	DB 79h			;
	DB 71h			;
	DB 76h			;
	DB 38h			;
	DB 6Dh			;
	DB 73h			;
;
L0068:  LXI D,R_111
	INX D
	LXI H,R_115
	MVI M,0D3H
	INX H
	INX H
	MVI M,0C9H
	DCX H
	MVI C,80H
L0096:  DCX D
	MOV A,C
	SUI 10H
	ORA A
	MOV C,A
	MOV M,A
	LDAX D
	CALL R_115
	CALL DELY1_m
	JNZ L0096
	CALL L0097
	RET
;
L0069:  CALL WRPROM
	RNC
	LXI H,034BH
	MOV B,A
	LDA R_105
	ANI 08H
	JZ L0099
	MOV A,B
	CPI 00H
	JZ L0099
	CPI 04H
	JZ L0099
	CPI 02H
	JZ L0099
	CPI 06H
	RNZ
L0190:  LDA R_105
	ANI 0F7H
	STA R_105
	LXI H,00A0H
	LDA 2FBAH
	ANI 0FH
	MOV B,A
L0099:  MOV A,B
	RLC
	ADD B
	MOV E,A
	MVI D,00H
	DAD D
	LXI D,032BH
	PUSH D
	PCHL
	CALL CASOT_m
	RET
;
WRPROM_m:  CALL L0100
	RNC
	MOV B,A
	CALL L0075
	CALL L0100
	JMP L0101
L0100:  IN 0BCH
	CALL L0077
	RNC
	CPI 07H
	JNZ L0076
	STC
	CMC
	RET
;
; kleine Sprungtabelle
;
	JMP L0102
	JMP L0103
	JMP L0104
	JMP L0105
	JMP L0106
	JMP L0071
	JMP L0107
;
RDPROM_m:  SHLD R_112
L0071:  LDA R_105
	ANI 0FDH
	ORI 01H
	JMP L0108
L0070:  LXI H,rambeg
L0017:  SHLD R_112
L0106:  MVI A,08H
	STA 2FB5H
	LDA R_105
	ANI 0F2H
	ORI 02H
	JMP L0108
L0105:  MVI A,48H
	STA 2FB5H
	LDA R_105
	ANI 0FEH
	ORI 06H
	JMP L0108
L0104:  MVI A,49H
	STA 2FB5H
	MVI A,07H
	STA 2FCAH
	XRA A
	STA 2FC9H
	MVI A,0FFH
	STA 2FBAH
	LXI H,0000H
	SHLD R_112
	LDA R_105
	ANI 0F9H
	ORI 09H
L0108:  STA R_105
L0066:  CALL L0109
	CALL L0073
	RET
;
L0102:  LDA R_105
	ANI 08H
	JNZ L0110
	CALL L0111
	JNZ L0112
	INX H
	SHLD R_112
	JMP L0066
L0112:  LDA R_106
	INR A
	DAA
L0114:  ANI 0FH
	STA R_106
	JMP L0066
L0110:  LXI H,2FCAH
	LDA 2FC9H
	CMP M
	RNC
	LHLD R_112
	XCHG
	LXI H,SRCBEG
	MOV C,A
	MVI B,00H
	DAD B
	MOV M,E
	INX H
	MOV M,D
	LXI H,0000H
	SHLD R_112
	LXI H,2FC9H
	INR M
	INR M
	JMP L0066
L0103:  CALL L0109
	JNZ L0113
	DCX H
	SHLD R_112
	JMP L0066
L0113:  LDA R_106
	DCR A
	JP L0114
	MVI A,09H
	JMP L0114
L0107:  XRA A
	STA R_108
	CALL CASOT_m
PRTASC_m:  LDA R_105
	ANI 80H
	CNZ RST1
	LDA REG_F
	MOV C,A
	PUSH B
	POP PSW
	LHLD REG_SP
	MOV A,L
	MOV L,H
	MOV H,A
	SPHL
	LXI H,REG_B
	MOV B,M
	INX H
	MOV C,M
	INX H
	MOV D,M
	INX H
	MOV E,M
	LHLD R_112
	PUSH H
	LHLD REG_H
	MOV A,L
	MOV L,H
	MOV H,A
	LDA REG_A
	RET
;
CASOT_m:  CALL L0116
	JC L0117
	CALL L0075
	CALL L0116
	RNC
L0117:  CALL L0068
	JMP CASOT_m
L0116:  CALL L0074
	RC
	CALL L0100
	RET
;
CASIN_m:  CALL DELY100_m
	CALL UHR_m
	JC L0118
	CALL WRPROM_m
	RNC
L0118:  CALL CASOT_m
	CALL L0070
	JMP WARMST
ONESEC_m:  PUSH PSW
	MVI A,0AH
L0120:  CALL DELY100_m
	DCR A
	JNZ L0120
	POP PSW
	RET
;
DELY100_m:  PUSH PSW
	MVI A,0AH
L0121:  PUSH PSW
	PUSH H
	PUSH D
	PUSH B
	CALL L0068
	POP B
	POP D
	POP H
	POP PSW
	DCR A
	JNZ L0121
	POP PSW
	RET
;
L0075:  PUSH PSW
	LDA 0007H
L0122:  CALL DELY1_m
	DCR A
	JNZ L0122
	POP PSW
	RET
L0097:  CALL DELY1_m
DELY1_m:  PUSH PSW
	LDA 0006H
	ADD A
	ADD A
	INR A
	ADD A
	ADD A
	ADD A
L0123:  DCR A
	JNZ L0123
	POP PSW
	RET
;
; Freiraum von 04BF bis 04FF
; 
;
; Seite 0500h
;
	org 0500h
	LHLD R_112
	MVI A,40H
	OUT SSM_FF
	PCHL
L0058:  LXI H,0003H
	CALL L0124
	LHLD R_112
	SHLD 2FD3H
	MOV A,M
	STA 2FD5H
	MVI M,0D7H
	LDA REG_F
	MOV C,A
	PUSH B
	POP PSW
	LHLD REG_SP
	MOV A,L
	MOV L,H
	MOV H,A
	SPHL
	LXI H,REG_B
	MOV B,M
	INX H
	MOV C,M
	INX H
	MOV D,M
	INX H
	MOV E,M
	LHLD SRCBEG
	PUSH H
	LHLD REG_H
	MOV A,L
	MOV L,H
	MOV H,A
	LDA REG_A
	RET
;
L0061:  LXI H,0000H
	LXI D,0003H
	DAD D
	CALL L0124
	RET
;
L0124:  SHLD 2FD1H
	MVI A,0C3H
	STA 2FD0H
	LDA R_105
	ANI 0F7H
	ORI 80H
	STA R_105
	XRA A
	STA REG_F
	LXI H,0000H
	SHLD 2FD3H
	SHLD 2FD6H
	LXI H,R_111
	MVI M,00H
	DCX H
	MVI M,50H
	DCX H
	MVI M,1CH
	DCX H
	MVI M,54H
	RET	
;
RST1_m:  LHLD R_112
	CALL L0125
	MOV A,M
	STA 2FD5H
	MVI M,0D7H
	LDAX D
	CPI 0E9H
	JNZ L0126
	LHLD REG_H
	MOV A,L
	MOV L,H
	MOV H,A
	JMP L0127
L0126:  CALL L0128
	JZ L0129
	LDAX D
	ANI 38H
	INR A
	MOV L,A
	MVI H,00H
	MOV E,M
	INX H
	MOV D,M
	XCHG
	JMP L0127
L0129:  CALL L0130
	JZ L0131
	LHLD REG_SP
	MOV A,L
	MOV L,H
	MOV H,A
	MOV E,M
	INX H
	MOV D,M
	XCHG
	JMP L0127
L0131:  CALL L0132
	RZ
	INX D
	LDAX D
	MOV L,A
	INX D
	LDAX D
	MOV H,A
L0127:  SHLD 2FD6H
	MOV A,M
	STA 2FD8H
	MVI M,0D7H
	RET
;
L0125:  PUSH H
	XCHG
	MVI B,03H
	CALL L0133
	JNZ L0134
	DCR B
	CALL L0135
	JNZ L0134
	DCR B
L0134:  MOV C,B
	MVI B,00H
	POP H
	DAD B
	SHLD 2FD3H
	RET
;
L0135:  LXI H,0650H
	MVI C,12H
	JMP L0136
L0133:  LXI H,0662H
	MVI C,1AH
	JMP L0136
L0132:  LXI H,066AH
	MVI C,12H
	JMP L0136
L0130:  LXI H,067CH
	MVI C,09H
	JMP L0136
L0128:  LXI H,0685H
	MVI C,08H
L0136:  LDAX D
	CMP M
	JZ L0137
	INX H
	DCR C
	JNZ L0136
L0137:  XRA A
	CMP C
	RET
;
RST2_m: SHLD R_115
	STA 2FCFH
	POP H
	DCX H
	SHLD R_112
	LHLD 2FD3H
	LDA 2FD5H
	MOV M,A
	LHLD 2FD6H
	LDA 2FD8H
	MOV M,A
	LDA 2FCFH
	LHLD R_115
	CALL PRTHEX
	LXI H,0000H
	SHLD 2FD3H
	SHLD 2FD6H
	PUSH H
	PUSH H
	POP H
	POP H
	LXI SP,STACK
	JMP L0139
	MVI B,0EH
	MVI D,1EH
	MVI H,2EH
	MVI M,3EH
	ADI 0CEH
	SUI 0DEH
	ANI 0EEH
	ORI 0FEH
	OUT 0DBH
	LXI B,2111H
	SHLD 312AH
	STA 0C23AH
	JMP L0140
	JC L0141
	JP L0142
	CZ L0143
	CC L0144
	CP L0145
	RZ	
	RET
;
; Tabelle ??
;
	DB 0D0h			; RNC
	DB 0D8h			; RC
	DB 0E0h			; RPO
	DB 0E8h			; RPE
	DB 0F0h			; RP
	DB 0F8h			; RM
	DB 0C7h			; RST 0
	DB 0CFh			; RST 1
	DB 0D7h			; RST 2
	DB 0DFh			; RST 3
	DB 0E7h			; RST 4
	DB 0EFh			; RST 5
	DB 0F7h			; RST 6
	DB 0FFh			; RST 7
;
L0053:  LHLD R_112
	SHLD DSTBEG
	CALL L0042
	LHLD DSTBEG
	JMP RDPROM
L0042:  CALL COPY
	LHLD SRCBEG
	XCHG
	LHLD DSTBEG
	SHLD 2FD3H
	MOV A,L
	SUB E
	MOV L,A
	MOV A,H
	SBB D
	MOV H,A
	SHLD DSTEND
	XCHG
	LHLD SRCEND
	DAD D
	SHLD 2FD6H
L0150:  LHLD 2FD3H
	SHLD R_115
	CALL L0125
	XCHG
	LHLD 2FD6H
	CALL L0148
	RNC
	LHLD R_115
	XCHG
	CALL L0133
	CNZ L0149
	JMP L0150
L0151:  XCHG
	LHLD SRCEND
L0148:  MOV A,D
	CMP H
	RC
	MOV A,E
	CMP L
	RET
;
L0149:  INX D
	LDAX D
	MOV L,A
	INX D
	LDAX D
	MOV H,A
	CALL L0151
	RNC
	LHLD SRCBEG
	CALL L0148
	JZ L0152
	RC
L0152:  LHLD DSTEND
	DAD D
	XCHG
	LHLD R_115
	INX H
	MOV M,E
	INX H
	MOV M,D	
	RET
;
L0054:  LDA R_112
	PUSH PSW
	MOV E,A
	MVI D,00H
	LHLD SRCBEG
	PUSH H
	DAD D
	SHLD DSTBEG
	LXI H,buff_th
	SHLD SRCEND
	CALL COPY_m
	POP H
	POP PSW
	PUSH H
L0153:  MVI M,00H
	INX H
	DCR A
	JNZ L0153
	POP H
	JMP RDPROM
L0060:  LDA R_112
	MOV E,A
	MVI D,00H
	LHLD SRCBEG
	SHLD DSTBEG
	PUSH H
	DAD D
	SHLD SRCBEG
	LXI H,buff_th
	SHLD SRCEND
	CALL COPY_m
	POP H
	JMP RDPROM
L0059:  LHLD R_112
	SHLD DSTBEG
	CALL COPY_m
	LHLD DSTBEG
	JMP RDPROM
COPY_m:  LHLD SRCEND
	XCHG
	LHLD DSTBEG
	CALL L0154
	MOV B,H
	MOV C,L
	LHLD SRCBEG
	JNC L0155
	MOV D,B
	MOV E,C
	CALL L0154
	PUSH H
	LHLD SRCEND
	XCHG
	POP H
	JNC L0155
	PUSH D
	PUSH H
	MOV A,L
	CMA
	MOV L,A
	MOV A,H
	CMA
	MOV H,A
	XCHG
	LHLD SRCEND
	DAD D
	XCHG
	LHLD DSTBEG
	DAD D
	INX H
	MOV B,H
	MOV C,L
	POP D
	POP H
L0157:  MOV A,M
	STAX B
	CALL L0156
	RZ
	DCX H
	DCX B
	JMP L0157
L0155:  MOV A,M
	STAX B
	CALL L0156
	RZ
	INX H
	INX B
	JMP L0155
L0156:  MOV A,L
	CMP E
	RNZ
	MOV A,H
	CMP D
	RET
;
L0154:  MOV A,H
	CMP D
	RC
	RNZ
	MOV A,L
	CMP E
	RET
;
L0057:  LDA R_112
	STA DSTBEG
	CALL FILL_M
	LHLD SRCBEG
	JMP RDPROM
FILL_M:  LHLD SRCEND
	XCHG
	LHLD SRCBEG
	LDA DSTBEG
	MOV B,A
L0158:  MOV M,B
	CALL L0156
	INX H
	RZ
	JMP L0158
MEMCHK_m:  LXI H,CPYBUF
	LXI D,00FDH
	MVI B,49H
L0159:  DAD D
	MOV C,M
	MOV M,B
	MOV A,M
	MOV M,C
	CMP B
	JZ L0159
	MOV A,H
	DCR A
	MVI B,0FFH
	RET
;
; Freiraum von 07E9 bis 07FF
;
; Seite 0800h
;
	org 0800h
L0055:  LHLD R_112
        XCHG
        LHLD SRCEND
        MOV C,L
        MOV B,H
        LHLD SRCBEG
        CALL WRITU_m
        JMP L0056
WRITU_m:  PUSH D
        PUSH B
        MOV A,H
        ORI 80H
        MOV H,L
        MOV L,A
        MVI C,08H
        MOV A,C
        OUT UHR_CTL			; Portadress Uhr
        CALL L0160
        PUSH H
        MVI L,00H
        CALL L0161
        POP H
        CALL L0162
        POP D
        XCHG
        MOV A,H
        ANI 07H
        DCR A
        CALL L0163
        CALL L0161
        POP D
        XCHG
        CALL L0162
        MVI A,0F4H
        OUT UHR_CTL			; Portadress Uhr
        RET
L0162:  MOV A,H
        CALL L0164
L0161:  MOV A,L
        CALL L0164
        RET
L0164:  PUSH PSW
        ANI 0FH
        CALL L0163
        POP PSW
        RRC
        RRC
        RRC
        RRC
        ANI 0FH
        CALL L0163
        RET
L0163:  OUT UHR_DAT			; Portadress Uhr
        MOV A,C
        OUT UHR_CTL
        ORI 02H
        OUT UHR_CTL
        ANI 0FDH
        OUT UHR_CTL
        MVI A,10H
        ADD C
        MOV C,A
        RET
L0056:  CALL L0043
        JMP L0056
READUR_m:  LXI H,buff_uhr
        PUSH H
        XCHG
        MVI C,0CH
        MVI A,08H
        OUT UHR_CTL
        CALL L0160
L0165:  MOV A,C
        OUT UHR_CTL
        PUSH H
        POP H
        IN UHR_DAT
        STAX D
        INX D
        MVI A,10H
        ADD C
        MOV C,A
        CPI 0DCH
        JNZ L0165
        POP H
        CALL L0166
        MVI A,0F4H
        OUT 19H
        RET
L0166:  PUSH H
        LXI B,0006H
        DAD B
        MOV B,M
        POP H
        PUSH H
        XCHG
        POP H
        PUSH H
        CALL L0167
        CALL L0168
        CALL L0167
        CALL L0168
        CALL L0167
        LDAX D
        ANI 03H
        MOV M,A
        INX D
        INX D
        INX H
        CALL L0167
        LDAX D
        ANI 03H
        MOV M,A
        INX D
        INX H
        INX H
        CALL L0167
        LDAX D
        ANI 01H
        MOV M,A
        INX D
        INX H
        CALL L0167
        CALL L0167
        DCX D
        MVI C,04H
L0169:  LDAX D
        MOV M,A
        DCX D
        DCX H
        DCR C
        JNZ L0169
        MOV A,B
        ANI 07H
        INR A
        MVI M,00H
        DCX H
        MOV M,A
        POP H
        RET
L0168:  LDAX D
        ANI 07H
        JMP L0170
L0167:  LDAX D
        ANI 0FH
L0170:  MOV M,A
        INX D
        INX H
        RET
L0043:  CALL READUR_m
        LXI D,R_109
        CALL L0171
        INX D
        CALL L0171
        CALL L0171
        LDA 2FB8H
        ORI 80H
        STA 2FB8H
        CALL DELY100
        XRA A
        STA R_108
        RET
L0171:  MOV B,M
        INX H
        MOV A,M
        INX H
        RLC
        RLC
        RLC
        RLC
        ORA B
        PUSH H
        CALL L0173
        POP H
        RET
TI_m:  CALL READUR_m
        XCHG
        DCX D
        LXI H,2FFFH
        MVI M,00H
        DCX H
        MVI M,68H
        CALL L0174
        DCX H
        MVI M,3AH
        CALL L0174
        DCX H
        MVI M,3AH
        CALL L0174
        DCX H
        MVI M,3BH
        LXI D,2FE7H
        CALL L0174
        DCX H
        MVI M,2EH
        LXI D,2FE5H
        CALL L0174
        DCX H
        MVI M,2EH
        LXI D,2FE1H
        CALL L0174
        DCX H
        MVI M,2CH
        DCX H
        PUSH H
        LXI H,098EH
        LDA 2FE4H
        ANI 07H
        RLC
        MOV E,A
        MVI D,00H
        DAD D
        MOV E,M
        INX H
        MOV D,M
        POP H
        MOV M,E
        DCX H
        MOV M,D
        RET
L0174:  CALL L0175
L0175:  DCX H
        INX D
        LDAX D
        ADI 30H
        CPI 3AH
        JNC L0176
        CPI 30H
        JNC L0177
L0176:  MVI A,2DH
L0177:  MOV M,A
        RET
;
; Wochentage ..oMiDiMoDrFaSoS
;
	DB 00h			;                 
	DB 00h                  ;
	DB 6Fh                 	;
	DB 4Dh                 	;
	DB 69h                 	;
	DB 44h                  ;
	DB 69h                  ;
	DB 4Dh                  ;
	DB 6Fh                  ;
	DB 44h                  ;
	DB 72h                  ;
	DB 46h                  ;
	DB 61h                  ;
	DB 53h                  ;
	DB 6Fh                  ;
	DB 53h   		;
;
V24OUT_m:  PUSH PSW		; Daten von der V24-Schnittstelle einlesen
        LXI D,R_109
        CALL L0173
        POP PSW
        STA R_108
        LXI H,0000H
        SHLD R_110
        SHLD 2FB8H
L0179:  CALL CASIN
        JMP L0179
V24IN_m:  PUSH H
        LXI D,R_110
        MOV A,L
        CALL L0173
        POP H
        MOV A,H
        CALL L0173
L0180:  LXI H,0000H
        SHLD R_109
        JMP L0179
DIPL2_m:  LXI D,R_110
        CALL L0173
        LXI H,R_111
        MVI M,79H
        DCX H
        MVI M,50H
        DCX H
        MVI M,50H
        JMP L0180
;
; Freiraum bis 09ffh
;	
	Org 0A00h
;	
L0062:  CALL CSTS_V
        JNZ L0062
        CALL L0063
        CALL L0066
L0184:  CALL L0182
        CALL L0068
        CALL L0183
        JMP L0184
L0182:  CALL CSTS_V
        RZ
        CALL CI
        MOV B,A
        MVI B,0AH
        CPI 82H
        JZ L0186
        MVI B,0BH
        CPI 8BH
        JZ L0186
        MVI B,0CH
        CPI 80H
        JZ L0186
        MVI B,0DH
        CPI 2FH
        JZ L0186
        MVI B,0EH
        CPI 89H
        JZ L0186
        MVI B,0FH
        CPI 8FH
        JZ L0186
        CALL HXTEST_V
        RZ
L0186:  MOV C,B
        MOV A,C
        CALL L0072
        CALL L0073
L0188:  CALL L0068
        CALL CSTS_V
        JNZ L0188
        RET
L0183:  LXI H,0AEDH
        PUSH H
        IN 0CH
        LXI H,2FCCH
        XRA M
        CPI 20H
        JZ L0104
        CALL CSTS_V
        POP H
        RZ
        PUSH H
        CALL CI
        MOV B,A
        LDA R_105
        ANI 08H
        MOV A,B
        JZ L0189
        CPI 0DH
        JZ L0190
        CPI 20H
        JZ L0190
        CPI 3CH
        JZ L0190
        CPI 2DH
        JZ L0190
        CPI 72H
        JZ L0190
        CPI 52H
        JZ L0190
        CPI 2BH
        JZ L0190
L0189:  CPI 3EH
        JZ L0071
        CPI 2AH
        JZ L0071
        CPI 3CH
        JZ L0106
        CPI 2DH
        JZ L0106
        CPI 72H
        JZ L0105
        CPI 52H
        JZ L0105
        CPI 2BH
        JZ L0105
        CPI 0DH
        JZ L0102
        CPI 20H
        JZ L0102
        CPI 5EH
        JZ L0103
        CPI 2EH
        JZ L0103
        CPI 67H
        JZ L0107
        CPI 47H
        JZ L0107
        CPI 3FH
        JZ L0107
        POP H
        RET
L0191:  CALL L0068
        CALL CSTS_V
        JNZ L0191
        RET
;
; Freiraum von 0AF7 bis 0AFF
;
	org 0B00h
;	
L0024:  PUSH D
        PUSH H
        LDA 005FH
        MOV B,A
        MVI A,0C2H
        LHLD 0016H
        JMP L0192
L0025:  PUSH D
        PUSH H
        MVI A,8AH
        OUT PAR_CTL			; Portadresse Paralleles Interface
        MVI A,01H
        OUT PAR_PC			; Portadresse Paralleles Interface
        LXI H,2E03H
        SHLD buff_th
        XRA A
        STA 2E02H
        MVI A,01H
        OUT 0BAH			; Portadresse Paralleles Interface
        LDA 005EH
        MOV B,A
        MVI A,0A0H
        LHLD 000EH
        JMP L0192
        JMP L0193			; --> INIT V24
        JMP L0194
        JMP RI_m
;	
L0193:  PUSH D				; INIT V24-Schnittstelle
        PUSH H
        LDA 0060H			;
        MOV B,A
        MVI A,58H
        LHLD 001EH
L0192:  OUT CS_MODE			; Modus ??
        MVI A,03H
        OUT ACIA_CTL			; Reset ACIA
        MOV A,B
        OUT ACIA_CTL			; Datenübertragung (bit, 1Stop, ohne Parität)
        MVI A,36H
        OUT TIMER_CTL			; Timer_0 Mode 3, erst unteres, dann oberes Byte
        CALL BD_HOL			; Bd-Werte ermitteln Rückgabe (HL)
        MOV A,L				; Startwerte für Baudrate laden
        OUT TIMER_0
        MOV A,H
        OUT TIMER_0
        POP H
        POP D
        RET
;	
BD_HOL: MOV A,H		; Timerwerte
        RLC
        MVI A,00H
        ADC H
        ANI 07H
        RLC
        MOV E,A
        MVI D,00H	; Zeiger innerhalb der Tabell
        LXI H,BD_T3	; Tabelle auswählen
        LDA 0006H	; (06h) <-- aus Adresse 0006h CPU-Takt
        CPI 07H
        JNC BD_END	; CPU-Takt > 7MHz
        LXI H,BD_T2
        CPI 05H
        JNC BD_END	; CPU-Takt > 5MHz < 7 Mhz
        LXI H,BD_T1	; CPU-Takt < 5 MHz
BD_END: DAD D
        MOV E,M		; Werte aus Tabelle holen
        INX H
        MOV D,M
        XCHG
        RET		; (HL)Timerwerte für Baudrate
;
; Baudratentabell 
; Quarz < 5Mhz 4 Mhz
BD_T1:  db 0b5h		; 4800 Bd* Standard
	db 01h		; 
	db 78h		; 110 Bd
	db 4Ah		;
	db 0D4h		; 1200 Bd
	db 06h		;
	db 4Eh		; 300Bd
	db 1Bh		;
	db 6Ah		; 2400 Bd
	db 03h		;
	db 0B5h		; 4800 Bd
	db 01h		;
	db 0A7h		; 600 Bd
	db 0Dh		; 
	db 0DAh		; 9600 Bd
	db 00h
;
; Quarz > 5 MHz < 7MHz , hier 6,144MHz	
BD_T2:	db 80h		; 
	db 02h
	db 17h
	db 6Dh
	db 0C4h
	db 09h
	db 00h
	db 28h
	db 00h
	db 05h
	db 80h
	db 02h
	db 00h
	db 14h
	db 40h
	db 01h 
;
; Quarz > 7Mhz 8MHz (8085A2 bis 10Mhz)
BD_T3:	db 41h		
	db 03h
	db 0Ch
	db 08Eh
	db 05h
	db 0Dh
	db 15h
	db 34h
	db 83h
	db 06h
	db 41h
	db 03h
	db 0Bh
	db 1Ah
	db 0A1h
	db 01h
;	
;	db 2A		; ????
;	db 0BBh
;	db 2Fh
;	db 22h
;	db 0C1h
;	db 2Fh
;
; oder diesen Code 
	lhld R_112	;
	shld SRCBEG	;
;
;
L0040:  LDA SRCBEG
        CPI 0ADH
        JZ L0200
        CALL L0024
        LXI B,0258H
L0202:  XRA A
        CALL L0201
        DCX B
        CMP C
        JNZ L0202
        CMP B
        JNZ L0202
        MVI A,0D3H
        CALL L0203
        CALL L0201
        XRA A
        CALL L0201
        LHLD SRCEND
        INX H
        XCHG
        LHLD SRCBEG
L0204:  MOV C,M
        CALL PO_m
        INX H
        MOV A,L
        CMP E
        JNZ L0204
        MOV A,H
        CMP D
        JNZ L0204
        XRA A
        CALL L0203
        CALL L0203
        CALL L0025
        RET
L0048:  LHLD R_112
        SHLD SRCEND
        CALL L0041
        RNZ
        JMP DIPL2
L0041:  CALL L0024
        MVI B,64H
L0207:  MVI C,0AH
L0209:  CALL RI_m
        CPI 00H
        JZ L0206
        DCR B
        JNZ L0207
        JMP L0208
L0206:  DCR C
        JNZ L0209
        JMP L0210
L0208:  MVI A,0C3H
        OUT CS_MODE
        MVI B,64H
L0212:  MVI C,0AH
L0214:  CALL RI_m
        CPI 00H
        JZ L0211
        DCR B
        JNZ L0212
        MVI A,00H
        JMP L0213
L0211:  DCR C
        JNZ L0214
L0210:  MVI C,03H
L0215:  CALL RI_m
        CPI 0D3H
        JNZ L0210
        DCR C
        JNZ L0215
        JMP L0216
        MVI A,01H
        JMP L0213
L0216:  CALL RI_m
        CPI 00H
        JZ L0217
        XRA A
        MVI A,02H
        JMP L0213
L0217:  LHLD SRCEND
        INX H
        XCHG
        LHLD SRCBEG
L0218:  CALL RI_m
        RZ
        MOV M,A
        INX H
        MOV A,L
        CMP E
        JNZ L0218
        MOV A,H
        CMP D
        JNZ L0218
        XRA A
        INR A
L0213:  PUSH PSW
        CALL L0025
        POP PSW
        RET
L0200:  MVI A,31H
        OUT TIMER_CTL
        MVI A,00H
        OUT TIMER_0
        MVI A,50H
        OUT TIMER_0
        MVI A,01H
        OUT CS_MODE
        XRA A
        OUT CS_MODE
        CALL L0160
        MVI A,01H
        OUT TIMER_CTL
        IN TIMER_0
        MOV L,A
        IN TIMER_0
        MOV H,A
        XCHG
        LHLD UPDATE
        PUSH H
        XCHG
        CALL ADROT_V
        POP H
        SHLD UPDATE
        CALL ONESEC_m
        JMP L0200
L0203:  CALL L0201
L0201:  PUSH B
        MOV C,A
        PUSH PSW
        CALL PO_m
        POP PSW
        POP B
        RET
RI_m:  PUSH B
        MVI C,0CH
L0222:  IN ACIA_CTL
        ANI 01H
        JNZ L0221
        CALL L0160
        DCR C
        JNZ L0222
        MVI A,03H
        POP B
        RET
L0221:  IN ACIA_DAT
        POP B
        RET
PO_m:  MOV B,C
        MVI C,0CH
L0224:  IN ACIA_CTL
        ANI 02H
        JNZ L0223
        CALL L0160
        DCR C
        JNZ L0224
        MVI A,04H
        RET
L0223:  MOV A,B
        OUT ACIA_DAT
        RET
;
LO_m:  LDA R_107		; Daten an Drucker mit 4800Bd
        ANI 0F0H
        RLC
        JC L0194
        RLC
        JC L0225
        RLC
        JC L0226
        RLC
        JC L0227
        XRA A
        MVI A,05H
        RET
L0194:  PUSH B
        LXI B,2000H
L0229:  IN ACIA_CTL
        ANI 02H
        JNZ L0228
        CALL L0160
        DCX B
        XRA A
        CMP B
        JNZ L0229
        CMP C
        JNZ L0229
        POP B
        MVI A,07H
        RET
L0228:  IN ACIA_CTL
        ANI 08H
        JNZ L0228
        LXI B,0000H
        PUSH B
        POP PSW
        POP B
        MOV A,C
        OUT ACIA_DAT
        RET
L0225:  PUSH B
        LXI B,2000H
L0231:  IN 0CAH
        ANI 40H
        JNZ L0230
        CALL L0160
        DCX B
        XRA A
        CMP B
        JNZ L0231
        CMP C
        JNZ L0231
        POP B
        MVI A,06H
        RET
L0230:  POP B
        MOV A,C
        OUT 0CAH
        OUT 0BAH
        RET
L0226:  PUSH B
        LXI B,2000H
L0233:  IN PAR_PC
        ANI 80H
        JZ L0232
        CALL L0160
        DCX B
        XRA A
        CMP B
        JNZ L0233
        CMP C
        JNZ L0233
        POP B
        MVI A,08H
        RET
L0232:  POP B
        MOV A,C
        OUT PAR_PA
        XRA A
        OUT PAR_PC
        CALL L0234
        INR A
        OUT PAR_PC
L0234:  RET
L0227:  MOV A,C
        CPI 08H
        JZ L0235
        CPI 0CH
        JZ L0236
        CPI 0DH
        JZ L0237
        CPI 7FH
        JZ L0235
        CPI 20H
        JC L0238
        CPI 0A0H
        JC L0239
L0238:  NOP
        MVI A,09H
        RET
L0239:  LHLD buff_th
        MOV M,C
        INX H
        SHLD buff_th
        LDA 2E02H
        INR A
        STA 2E02H
        CPI 14H
        RNZ
L0237:  XRA A
        STA 2E02H
        LXI H,2E03H
        SHLD buff_th
        CALL L0240		; Sprung nach Adr 2014h Thermodrucker ?
L0236:  MVI A,14H
        LXI H,2E03H
L0241:  MVI M,20H
        INX H
        DCR A
        JNZ L0241
        INR A
        RET
L0235:  LDA 2E02H
        DCR A
        RM
        STA 2E02H
        LHLD buff_th
        DCX H
        MVI M,20H
        SHLD buff_th
        RET
L0049:  LHLD R_112
        INX H
        XCHG
        LHLD SRCBEG
L0243:  MOV C,M
        CALL L0242
        INX H
        MOV A,L
        CMP E
        JNZ L0243
        MOV A,H
        CMP D
        JNZ L0243
        RET
L0050:  LHLD R_112
        INX H
        XCHG
        LHLD SRCBEG
        MVI A,0C0H
        SIM
L0245:  CALL L0244
        MOV M,A
        INX H
        MOV A,L
        CMP E
        JNZ L0245
        MOV A,H
        CMP D
        JNZ L0245
        RET
L0242:  RIM
        ANA A
        JP L0242
        MOV A,C
        CALL L0246
        MVI C,08H
L0250:  RRC
        ANA A
        JM L0247
        CALL L0246
        JMP L0248
L0247:  CALL L0249
L0248:  DCR C
        JNZ L0250
        CALL L0249
        RET
L0246:  PUSH PSW
        MVI A,0C0H
        JMP L0251
L0249:  PUSH PSW
        MVI A,40H			; Portadresse FDC
L0251:  SIM
        CALL L0252
        POP PSW
        RET
L0244:  RIM
        ANA A
        JP L0244
        PUSH D
        MVI D,08H
        MVI E,00H
        CALL L0160
L0253:  CALL L0252
        RIM
        ANI 80H
        ORA E
        RRC
        MOV E,A
        DCR D
        JNZ L0253
        RLC
        PUSH PSW
        CALL L0252
        POP PSW
        POP D
        RET
L0252:  CALL L0160
        CALL L0160
        RET
L0051:  LHLD R_112
        XCHG
        LHLD SRCBEG
        MVI A,40H			; Portadresse FDC
        OUT FDC_SEL
        CALL FDCK
        XRA A
        OUT FDC_SEL
        INR A
        RET
L0052:  LHLD R_112
        SHLD SRCEND
        LHLD SRCBEG
        MVI A,40H
        OUT FDC_SEL
        CALL FDCW
        XRA A
        OUT FDC_SEL
        INR A
        RET
        RST 7
        RST 7
        RST 7
        RST 7
        RST 7
        RST 7
        RST 7
CO_m:  PUSH PSW
        PUSH H
        PUSH D
        PUSH B
        MVI A,0C3H
        STA 2FD3H
        LXI H,1006H
        SHLD 2FD4H
        LDA R_105
        MOV B,A
        ANI 10H
        JNZ L0256
        MOV A,C
        CPI 1BH
        JZ L0257
        MOV A,B
        JMP L0258
L0256:  MOV A,B
        ANI 40H
        JZ L0259
        MOV A,C
        STA 2FD7H
        MVI C,16H
        JMP L0260
L0259:  MOV A,B
        ANI 20H
        JZ L0261
        MOV A,C
        STA 2FD6H
        MOV A,B
        ORI 40H
        JMP L0262
L0261:  MOV A,C
        CPI 16H
        JNZ L0260
        MOV A,B
        ORI 20H
        JMP L0262
L0260:  MOV A,C
        CPI 20H
        JNC L0263
        CPI 1AH
        JNC L0264
        CPI 10H
        JC L0264
        LXI H,0F28H
        PUSH H
        LXI H,0F43H
        SUI 10H
        RLC
        MOV E,A
        MVI D,00H
        DAD D
        MOV E,M
        INX H
        MOV D,M
        XCHG
        PCHL
L0263:  LDA R_105
        ANI 8FH
        JMP L0258
L0264:  LDA R_105
        ANI 8FH
L0262:  LXI H,0F98H
        SHLD 2FD4H
L0258:  STA R_105
        POP B
        POP D
        POP H
        POP PSW
        JMP L0265
L0257:  MOV A,B
        ORI 10H
        JMP L0262
        DCR L
        DB 10H 			;UNRECOGNIZED
        MOV D,A
        RRC
        DAD SP
        DB 10H 			;UNRECOGNIZED
        MVI M,10H
        MOV E,L
        RRC
        MOV A,E
        RRC
        SBB C
        RRC
        XRA M
        RRC
        ORA A
        RRC
        CMP L
        RRC
        LXI H,3140H
        JMP CRSPOS_V
        LXI B,3140H
        LXI D,3780H
        LXI H,3190H
L0267:  IN VID_DSPEN
        RRC
        JNC L0267
        MOV A,M
        STAX B
        INX B
        INX H
        MOV A,E
        CMP L
        JNZ L0267
        MOV A,D
        CMP H
        JNZ L0267
        RET
        LXI B,VIDEND
        LXI D,313FH
        LXI H,372FH
L0268:  IN VID_DSPEN
        RRC
        JNC L0268
        MOV A,M
        STAX B
        DCX B
        DCX H
        MOV A,E
        CMP L
        JNZ L0268
        MOV A,D
        CMP H
        JNZ L0268
        RET
        LHLD 2FD6H
        XCHG
        LXI H,2FB0H
        LXI B,0050H
L0269:  DAD B
        DCR E
        JNZ L0269
        MOV C,D
        DAD B
        CALL CRSPOS_V
        RET
        LXI D,3780H
        LHLD UPDATE
        JMP L0270
        CALL L0271
        JMP CLRLIN_V
        CALL L0271
        LXI B,0050H
        DAD B
        XCHG
        LHLD UPDATE
L0270:  IN VID_DSPEN
        RRC
        JNC L0270
        MVI M,20H
        INX H
        MOV A,E
        CMP L
        JNZ L0270
        MOV A,D
        CMP H
        JNZ L0270
        RET
L0271:  LXI B,0FFB0H
        LXI D,3780H
        LHLD UPDATE
        XCHG
L0275:  MOV A,H
        CMP D
        JC L0273
        JNZ L0274
        MOV A,L
        CMP E
        JC L0273
        JZ L0273
L0274:  DAD B
        JMP L0275
L0273:  RET
        
end	
;---------------------------------------------
; LIST OF LABELS
; Sorted by name:
;---------------------------------------------
;
; Interuptsprugverteiler
;
RST3_m:		equ   	2F8Fh	;
RST4_m:		equ   	2F92h	;
TRAP_m: 	equ   	2F8Ch	;
RST5_m: 	equ   	2F95h	;
RST55_m: 	equ   	2F98h	;
RST6_m: 	equ   	2F9Bh	;
RST65_m: 	equ   	2F9Eh	;
RST7_m: 	equ   	2FA1h	;
RST75_m: 	equ   	2FA4h	;
;
; zum Video-Monitor
;
KALT_V: 	equ   	1000h	; kalt: Videomonitor Kaltstart
CI_V: 		equ   	1009h	; Videomonitor Tastaurabfrage
CSTS_V: 	equ   	100Ch	; Videomonitor Tastaturstatus
HXTEST_V: 	equ	1015h	; hxtest: Videomonitor ASCII/HEX-Umsetzung
CLRLIN_V: 	equ  	1030h	; clrlin: Videomonitor Bildzeile löschen
CRSPOS_V: 	equ  	1033h	; crspos: Videomonitor Cursor positionieren
ADROT_V: 	equ  	103Ch	; adrot: Videomonitor (H&L) ausgeben
;
; zum FDC
;
FDCK: 		equ	2000h	; FDCK: FDC-Routinen Kaltstart
FDCW: 		equ	2003h	; WARM: FDC-Routinen Warmstart

L0240: 		equ	2014h	; Sprung nach Adr 2014h Thermodrucker ? oder FDC-Routinen

L0139: 		equ  	2FD0h	;
L0265: 		equ  	2FD3h	;

DIPL4_m: 	equ   	0BB6h	; ? 0BB5 01 2A BB            LXI B,0BB2AH
L0140: 		equ  	0D2CAh	;
L0141: 		equ  	0EAE2h	;
L0142: 		equ  	0C4FAh	;
L0143: 		equ  	0D4CDh	;
L0144: 		equ  	0ECE4h	;
L0145: 		equ  	0C0FCh	;
L0160: 		equ  	000Bh	;


L0197: 		equ  	4E06h
L0198: 		equ  	CPYBUF	; Kopierbuffer ab 8000h
L0199: 		equ  	9h

